package org.apache.bcel.classfile;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import org.apache.commons.lang3.ArrayUtils;

/* loaded from: org_apache_bcel_classfile.jar:org/apache/bcel/classfile/Code.class */
public final class Code extends Attribute {
    private int maxStack;
    private int maxLocals;
    private byte[] code;
    private CodeException[] exceptionTable;
    private Attribute[] attributes;

    public Code(Code c) {
        this(c.getNameIndex(), c.getLength(), c.getMaxStack(), c.getMaxLocals(), c.getCode(), c.getExceptionTable(), c.getAttributes(), c.getConstantPool());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public Code(int nameIndex, int length, DataInput file, ConstantPool constantPool) throws IOException {
        this(nameIndex, length, file.readUnsignedShort(), file.readUnsignedShort(), (byte[]) null, (CodeException[]) null, (Attribute[]) null, constantPool);
        int codeLength = file.readInt();
        this.code = new byte[codeLength];
        file.readFully(this.code);
        int exceptionTableLength = file.readUnsignedShort();
        this.exceptionTable = new CodeException[exceptionTableLength];
        for (int i = 0; i < exceptionTableLength; i++) {
            this.exceptionTable[i] = new CodeException(file);
        }
        int attributesCount = file.readUnsignedShort();
        this.attributes = new Attribute[attributesCount];
        for (int i2 = 0; i2 < attributesCount; i2++) {
            this.attributes[i2] = Attribute.readAttribute(file, constantPool);
        }
        super.setLength(length);
    }

    public Code(int nameIndex, int length, int maxStack, int maxLocals, byte[] code, CodeException[] exceptionTable, Attribute[] attributes, ConstantPool constantPool) {
        super((byte) 2, nameIndex, length, constantPool);
        this.maxStack = maxStack;
        this.maxLocals = maxLocals;
        this.code = code != null ? code : ArrayUtils.EMPTY_BYTE_ARRAY;
        this.exceptionTable = exceptionTable != null ? exceptionTable : CodeException.EMPTY_CODE_EXCEPTION_ARRAY;
        this.attributes = attributes != null ? attributes : EMPTY_ARRAY;
        super.setLength(calculateLength());
    }

    @Override // org.apache.bcel.classfile.Attribute, org.apache.bcel.classfile.Node
    public void accept(Visitor v) {
        v.visitCode(this);
    }

    private int calculateLength() {
        int len = 0;
        if (this.attributes != null) {
            for (Attribute attribute : this.attributes) {
                len += attribute.getLength() + 6;
            }
        }
        return len + getInternalLength();
    }

    @Override // org.apache.bcel.classfile.Attribute
    public Attribute copy(ConstantPool constantPool) {
        Code c = (Code) clone();
        if (this.code != null) {
            c.code = (byte[]) this.code.clone();
        }
        c.setConstantPool(constantPool);
        c.exceptionTable = new CodeException[this.exceptionTable.length];
        Arrays.setAll(c.exceptionTable, i -> {
            return this.exceptionTable[i].copy();
        });
        c.attributes = new Attribute[this.attributes.length];
        Arrays.setAll(c.attributes, i2 -> {
            return this.attributes[i2].copy(constantPool);
        });
        return c;
    }

    @Override // org.apache.bcel.classfile.Attribute
    public void dump(DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.maxStack);
        file.writeShort(this.maxLocals);
        file.writeInt(this.code.length);
        file.write(this.code, 0, this.code.length);
        file.writeShort(this.exceptionTable.length);
        for (CodeException exception : this.exceptionTable) {
            exception.dump(file);
        }
        file.writeShort(this.attributes.length);
        for (Attribute attribute : this.attributes) {
            attribute.dump(file);
        }
    }

    public Attribute[] getAttributes() {
        return this.attributes;
    }

    public byte[] getCode() {
        return this.code;
    }

    public CodeException[] getExceptionTable() {
        return this.exceptionTable;
    }

    private int getInternalLength() {
        return 8 + this.code.length + 2 + (8 * (this.exceptionTable == null ? 0 : this.exceptionTable.length)) + 2;
    }

    public LineNumberTable getLineNumberTable() {
        for (Attribute attribute : this.attributes) {
            if (attribute instanceof LineNumberTable) {
                return (LineNumberTable) attribute;
            }
        }
        return null;
    }

    public LocalVariableTable getLocalVariableTable() {
        for (Attribute attribute : this.attributes) {
            if (attribute instanceof LocalVariableTable) {
                return (LocalVariableTable) attribute;
            }
        }
        return null;
    }

    public int getMaxLocals() {
        return this.maxLocals;
    }

    public int getMaxStack() {
        return this.maxStack;
    }

    public void setAttributes(Attribute[] attributes) {
        this.attributes = attributes != null ? attributes : EMPTY_ARRAY;
        super.setLength(calculateLength());
    }

    public void setCode(byte[] code) {
        this.code = code != null ? code : ArrayUtils.EMPTY_BYTE_ARRAY;
        super.setLength(calculateLength());
    }

    public void setExceptionTable(CodeException[] exceptionTable) {
        this.exceptionTable = exceptionTable != null ? exceptionTable : CodeException.EMPTY_CODE_EXCEPTION_ARRAY;
        super.setLength(calculateLength());
    }

    public void setMaxLocals(int maxLocals) {
        this.maxLocals = maxLocals;
    }

    public void setMaxStack(int maxStack) {
        this.maxStack = maxStack;
    }

    @Override // org.apache.bcel.classfile.Attribute
    public String toString() {
        return toString(true);
    }

    public String toString(boolean verbose) {
        StringBuilder buf = new StringBuilder(100);
        buf.append("Code(maxStack = ").append(this.maxStack).append(", maxLocals = ").append(this.maxLocals).append(", code_length = ").append(this.code.length).append(")\n").append(Utility.codeToString(this.code, super.getConstantPool(), 0, -1, verbose));
        if (this.exceptionTable.length > 0) {
            buf.append("\nException handler(s) = \n").append("From\tTo\tHandler\tType\n");
            for (CodeException exception : this.exceptionTable) {
                buf.append(exception.toString(super.getConstantPool(), verbose)).append("\n");
            }
        }
        if (this.attributes.length > 0) {
            buf.append("\nAttribute(s) = ");
            for (Attribute attribute : this.attributes) {
                buf.append("\n").append(attribute.getName()).append(":");
                buf.append("\n").append(attribute);
            }
        }
        return buf.toString();
    }
}
